/**
 * Copyright (C) 2007 by Levi Bard
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * @author Levi Bard <taktaktaktaktaktaktaktaktaktak@gmail.com>
 */

#include "lightmediascanner_parser.h"

typedef struct {
	lms_plugin_t *plugin;
	lms_t *parent;
} ParserRuby;

/**
 *
 * Native memory deletion function 
 *
 * * return nil
 *
 */
static VALUE parser_free(ParserRuby *parser) {
	if(parser) {
		if(parser->plugin) {
			lms_parser_del(parser->parent, parser->plugin);
		}// delete plugin

		free(parser);
	}

	return Qnil;
}// parser_free

#if 0
// Obsolete
/**
 *
 * Add parser plugin, given its shared object path.
 *
 * * p1 (name) path to shared object (usable by dlopen(3)).
 *
 * * return A new Parser
 *
 */
static VALUE scanner_add_parser(VALUE self, VALUE name) {
	lms_t *lms;
	lms_plugin_t *plugin = NULL;
	char *cname = NULL;

	Check_Type(name, T_STRING);

	Data_Get_Struct(self, lms_t, lms);
	cname = StringValuePtr(name);

	if(!(plugin = lms_parser_add(lms, cname))) {
		plugin = lms_parser_find_and_add(lms, cname);
		if(!plugin) {
			rb_raise(rb_eRuntimeError, "Unable to load plugin '%s'", cname);
		}// throw!
	}// search

	//wrap and return plugin
	return lightmediascanner_parser_new(lms, plugin);
}// scanner_add_parser
#endif

/**
 *
 * Parser#new
 *
 * constructor
 *
 * * p1 (parent) The Scanner to which the parser belongs
 *
 * * p2 (name) path to shared object (usable by dlopen(3)).
 *
 * * return A new Parser
 *
 */
static VALUE parser_new(VALUE obj, VALUE parent, VALUE name) {
	lms_t *lms = NULL;
	char *cname = NULL;
	ParserRuby *parser = NULL;
	lms_plugin_t *plugin = NULL;

	Check_Type(name, T_STRING);

	Data_Get_Struct(parent, lms_t, lms);
	cname = StringValuePtr(name);

	if(!(plugin = lms_parser_add(lms, cname))) {
		plugin = lms_parser_find_and_add(lms, cname);
		if(!plugin) {
			rb_raise(rb_eRuntimeError, "Unable to load plugin '%s'", cname);
		}// throw!
	}// search

	parser = (ParserRuby*)malloc(sizeof(ParserRuby));
	parser->parent = lms;
	parser->plugin = plugin;

	return Data_Wrap_Struct(obj, NULL, parser_free, parser);
}// scanner_new

/**
 * LightMediaScanner::Parser is a class to represent 
 * a LightMediaScanner plugin.
 */
void Init_lightmediascanner_parser() {
	lightmediascanner_parser = rb_define_class_under(lightmediascanner_module, "Parser", rb_cObject);
	rb_define_singleton_method(lightmediascanner_parser, "new", parser_new, 2);
}// Init_lightmediaparser_parser
